# Probing the limit of hydrologic predictability with the Transformer network

This repository contains the code for our paper published in the Journal of Hydrology: [Probing the limit of hydrologic predictability with the Transformer network](https://doi.org/10.1016/j.jhydrol.2024.131389)

Citation:
```
Liu, J., Bian, Y., Lawson, K., & Shen, C. (2024). Probing the limit of hydrologic predictability with the Transformer network. Journal of Hydrology, 637, 131389.
```

## Project Structure

- `config/`: Contains configuration files for the model, data, and training
- `core/`: Contains a trainer framework for training and evaluating models
- `datasets/`: Contains a list of 531 selected watersheds
- `model/`: Contains the definition of the Transformer model
- `utils/`: Contains various utility functions
  - `utils/data`: For downloading, processing, and loading data
  - `utils/losses`: For defining loss functions
  - `utils/stats`: For calculating statistical metrics
- `examples/`: Contains example code
  - `examples/tutorial_download_data.py`: Demonstrates how to automatically download CAMELS and extended data
  - `examples/tutorial_Transformer.py`: Demonstrates how to train the Transformer model

## Environment Setup and Dependency Installation

Create and activate a conda environment:
```bash
conda create -n Transformer_paper python=3.8
conda activate Transformer_paper
```

Install project dependencies:
```bash
cd ./MHPI_Transformer
pip install -e .
```

## Usage Guide

### Data Download
Run the following command to download the required data:
```bash
python tutorial_download_data.py
```

### Training the Model
Run the following command to start training the Transformer model:
```bash
python tutorial_Transformer.py
```
Please note that different GPU types and CUDA versions may lead to slightly different results.
## License
This project's code is under the [Non-Commercial Software License](https://github.com/mhpi/hydroDL/blob/release/LICENSE)

## Acknowledgements
The implementation of this project references and modifies the following open-source projects:

- CAMELS data processing and loss functions: https://zenodo.org/records/4738770
- Framework:
  - https://github.com/qubvel-org/segmentation_models.pytorch
  - https://github.com/mhpi/hydroDL
- Transformer model: https://github.com/timeseriesAI/tsai
- Other inspirations: 
  - https://github.com/zhouhaoyi/Informer2020
  - https://github.com/thuml/autoformer

We thank the contributors of these projects.

## Funding
JL is supported by National Science Foundation EAR-2221880 and the U.S. Department of Energy under award DE-SC0016605. KL and CS are supported by Cooperative Institute for Research to Operations in Hydrology (CIROH) through the NOAA Cooperative Agreement with The University of Alabama (
NA22NWS4320003
) and subaward A22-0307-S003.