import os
import sys
import argparse
import warnings
import yaml
import re

class DefaultConfig(object):
    __name__ = "default configuration"

    def __init__(self, sys_argv=None):
        if sys_argv is None:
            sys_argv = sys.argv[1:]

        parser = argparse.ArgumentParser()

        # hyperparameters
        parser.add_argument("--BATCH_SIZE",
                            help="selected pixels in the mini_batch",
                            default=None,
                            type=int)
        parser.add_argument("--RHO",
                            help="selected date in the mini_batch",
                            default=None,
                            type=int)
        parser.add_argument("--EPOCH",
                            help="epoch numbers",
                            default=None,
                            type=int)
        parser.add_argument("--LR",
                            help="learning rate",
                            default=None,
                            type=float)
        parser.add_argument("--SEED",
                            help="For experiment reproduction ",
                            default=None,
                            type=int)
        parser.add_argument("--DROPOUT",
                            help="dropout rate for LSTM, from 0 to 1",
                            default=None,
                            type=float)
        parser.add_argument("--HIDDENSIZE",
                            help="hidden layer in the LSTM network",
                            default=None,
                            type=int)
        parser.add_argument("--BATCH_FIRST",
                            help="when generate dataset and input LSTM, the data dimension. (pixel, time, features)",
                            default=True,
                            type=bool)
        parser.add_argument("--DEVICE",
                            help="use GPU or CPU",
                            default="cuda",
                            type=str)
        parser.add_argument("--GPU_list",
                            help="which GPU is used",
                            default=[0],
                            type=list)
        parser.add_argument("--model_file",
                            help="Model location for retrain or prediction. e.g. /data/model/Epoch30.pt",
                            default=None,
                            type=str)

        # Transformer
        parser.add_argument("--Q",
                            help="Query size",
                            default=None,
                            type=int)
        parser.add_argument("--V",
                            help="Value size",
                            default=None,
                            type=int)
        parser.add_argument("--H",
                            help="Number of heads",
                            default=None,
                            type=int)
        parser.add_argument("--N_LAYERS",
                            help="Number of encoder/decoder layers",
                            default=None,
                            type=int)
        parser.add_argument("--ATTENTION_SIZE",
                            help="Attention window size",
                            default=None,
                            type=int)
        parser.add_argument("--pe",
                            help="Position encoding",
                            default=None,
                            type=int)
        parser.add_argument("--chunk_size",
                            help="Chunk size",
                            default=None,
                            type=int)
        parser.add_argument("--dim_feedforward",
                            help="hidden size for TR",
                            default=None,
                            type=int)
        parser.add_argument("--conv_kernel",
                            help="length of convolutional kernel",
                            default=0,
                            type=int)
        parser.add_argument("--CUDA_ID",
                            help="GPU to run on",
                            default=1,
                            type=int)

        # dataset setting
        parser.add_argument("--output_path",
                            help="",
                            default="./",
                            type=str)
        parser.add_argument("--rename_output",
                            help="using hyperparameter to rename output: "
                                 "EPOCH_100_RHO_365_BATCHSIZE_032_HIDDENSIZE_128_DROPOUT_0.5_train_20160401_20210331_valid_20150401_20160331",
                            default=True,
                            type=bool)
        parser.add_argument("--csv_path",
                            help="input data path",
                            default=None,
                            type=str)
        parser.add_argument("--train_date_list",
                            help="[str_date, end_date]",
                            default=None,
                            type=list)
        parser.add_argument("--valid_date_list",
                            help="validation or testing. [str_date, end_date]",
                            default=None,
                            type=list)
        parser.add_argument("--var_times",
                            help="timeseries variables list",
                            default=None,
                            type=list)
        parser.add_argument("--var_const",
                            help="attribute variables list",
                            default=None,
                            type=list)
        parser.add_argument("--target",
                            help="target variable list (timeseries)",
                            default=None,
                            type=list)
        parser.add_argument("--DI",
                            help="Data Integration",
                            default=None,
                            type=list)
        parser.add_argument("--multiforcing",
                            help="Use multiple forcings",
                            default=False,
                            type=bool)

        # saving setting
        parser.add_argument("--SAVE_EACH",
                            help="save the model for each epoch",
                            default=False,
                            type=bool)
        parser.add_argument("--logs_name",
                            help="record metrics after being tested",
                            default="test_metrics.json",
                            type=str)
        parser.add_argument("--pred_folder_name",
                            help="The folder storing the predicted csv results",
                            default="prediction",
                            type=str)

        # multi-scale
        parser.add_argument("--site_path",
                            help="site csv input data path",
                            default=None,
                            type=str)
        parser.add_argument("--grid_path",
                            help="grid csv input data path",
                            default=None,
                            type=str)
        parser.add_argument("--sampling_site_ratio",
                            help="sampling_site_ratio = site : (grid + site)",
                            default=0.5,
                            type=float)
        parser.add_argument("--sampling_base_on",
                            help="total sampling number at each epoch",
                            default=0.5,
                            type=float)
        parser.add_argument("--loss_ratio_site",
                            help="total loss = loss_site * loss_ratio_site + loss_grid * (1-loss_grid)",
                            default=0.5,
                            type=float)
        parser.add_argument("--site_target",
                            help="site target list",
                            default=None,
                            type=list)
        parser.add_argument("--grid_target",
                            help="grid target list",
                            default=None,
                            type=list)
        parser.add_argument("--use_batchnorm",
                            help="use_batchnorm",
                            default=False,
                            type=bool)
        parser.add_argument("--embConstruct",
                            help="embConstruct",
                            default=0,
                            type=int)

        # K-Fold
        parser.add_argument("--nth_kfold",
                            help="cross-validation for nth kfold",
                            default=None,
                            type=int)
        parser.add_argument("--KFOLD",
                            help="cross-validation for k-fold total number",
                            default=None,
                            type=int)

        # PUR
        parser.add_argument("--train_region_list",
                            help="Training data set regions, support bx, shapefile",
                            default=None,
                            type=list)
        parser.add_argument("--test_region_list",
                            help="Testing data set regions, support bx, shapefile",
                            default=None,
                            type=list)
        parser.add_argument("--PUR",
                            help="",
                            default=False,
                            type=bool)

        # Multiple-GPU
        parser.add_argument("--accelerator",
                            help="",
                            default=None,)

        # parser.add_argument("--",
        #                     help="",
        #                     default=None,
        #                     type=)

        args = parser.parse_args(sys_argv)
        print("Parsed arguments:", args)
        self.args = args

        # hyperparameter
        self.BATCH_SIZE = args.BATCH_SIZE
        self.RHO = args.RHO
        self.EPOCH = args.EPOCH
        self.LR = args.LR
        self.SEED = args.SEED
        self.DROPOUT = args.DROPOUT
        self.HIDDENSIZE = args.HIDDENSIZE
        self.BATCH_FIRST = args.BATCH_FIRST
        self.DEVICE = args.DEVICE
        self.GPU_list = args.GPU_list
        self.model_file = args.model_file

        # Transformer
        self.Q = args.Q
        self.V = args.V
        self.H = args.H
        self.N_LAYERS = args.N_LAYERS
        self.ATTENTION_SIZE = args.ATTENTION_SIZE
        self.pe = args.pe
        self.chunk_size = args.chunk_size
        self.dim_feedforward = args.dim_feedforward
        self.conv_kernel = args.conv_kernel

        # dataset
        self.output_path = args.output_path
        self.rename_output = args.rename_output
        self.csv_path = args.csv_path
        self.train_date_list = args.train_date_list
        self.valid_date_list = args.valid_date_list
        self.var_times = args.var_times
        self.var_const = args.var_const
        self.target = args.target
        self.DI = args.DI

        # saving setting
        self.SAVE_EACH = args.SAVE_EACH
        self.logs_name = args.logs_name
        self.pred_folder_name = args.pred_folder_name

        # multi-scale
        self.site_path = args.site_path
        self.grid_path = args.grid_path
        self.sampling_site_ratio = args.sampling_site_ratio
        self.sampling_base_on = args.sampling_base_on
        self.loss_ratio_site = args.loss_ratio_site
        self.site_target = args.site_target
        self.grid_target = args.grid_target

        # K-Fold
        self.KFOLD = args.KFOLD
        self.nth_kfold = args.nth_kfold

        # PUR: prediction in ungauged regions
        self.PUR = args.PUR
        self.train_region_list = args.train_region_list
        self.test_region_list = args.test_region_list

        self.test_region_list = args.test_region_list

        # Multiple-GPU
        self.accelerator = args.accelerator

        self.config_dict = vars(args)

    # update parameters
    def update(self, **kwargs):
        '''
        only update one param! don't cal with param in the class!!
        the number of k,v,h according to the param you need to update
        '''
        for k, v in kwargs.items():
            if not hasattr(self, k):
                # warnings.warn("There is no argument: {} !".format(k))
                raise ValueError("There is no argument: {} !".format(k))
            # update instance variable
            setattr(self, k, v)
            # update namespace
            setattr(self.args, k, v)
            # update config_dict
            self.config_dict[k] = v

        return self.config_dict

    def update_yml(self, path_yml, path_log, tag=None):
        pattern = re.compile('.*?\${(\w+)}.*?')
        loader = yaml.SafeLoader
        loader.add_implicit_resolver(tag, pattern, None)

        def constructor_env_variables(loader, node):
            value = loader.construct_scalar(node)
            match = pattern.findall(value)
            if match:
                full_value = value
                for g in match:
                    full_value = full_value.replace(f'${{{g}}}', path_log)
                return full_value
            return value

        temp = constructor_env_variables
        loader.add_constructor(tag, temp)
        with open(path_yml, encoding='utf-8') as conf_data:
            return yaml.load(conf_data, Loader=loader)


default_config = DefaultConfig()
