import numpy as np
import pandas as pd

import numpy as np
from waterGPT.utils.stats.transform import align_dim

def data_integration(x, y, DI=0, time_dim=1, remove_nan=True, fill_num=0):
    """

    Syntax:
        x = np.arange(1 * 5 * 1).reshape(1, 5, 1)
        y = np.arange(1 * 5).reshape(1, 5, 1)
        DI = 2
        x, y = data_integration(x, y, DI=DI, time_dim=1)
    output:
        x = array([[[2, 0],
                    [3, 1],
                    [4, 2]]])
        y = array([[[2],
                    [3],
                    [4]]])

    """
    if DI == 0:
        return x, y

    x = align_dim(x, dim=3)
    y = align_dim(y, dim=3)

    num_time_x = x.shape[time_dim]
    num_time_y = y.shape[time_dim]

    assert num_time_x == num_time_y and num_time_x > 3

    y_append = y[:, :num_time_y - DI, :]
    x_cut = x[:, DI:, :]
    y_cut = y[:, DI:, :]

    # remove nan with 0
    if remove_nan:
        y_append[np.isnan(y_append)] = fill_num

    x_merged = np.concatenate([x_cut, y_append], axis=-1)

    return x_merged, y_cut

def shift_date(DI, date_list):

    """
    DI = 2
    date_list = ["2000-01-01", "2000-01-31"]
    Syntax:
        new_date_list = shift_date(DI, date_list)
        new_date_list = ["2000-01-03", "2000-01-31"]
    """

    str_date, end_date = date_list
    date_range = pd.date_range(str_date, end_date)
    date_range_shift = date_range.shift(DI)
    str_date = date_range_shift[0]
    str_date = str_date.strftime("%Y-%m-%d")

    assert len(pd.date_range(str_date, end_date)) > 3

    return [str_date, end_date]






























