from torch.utils.data.dataset import Dataset as DS
import numpy as np
import torch

class Dataset(DS):

    def __init__(
            self,
            x_data,
            y_data,
            batch_size,
            rho,
            stage_name="train",
            batch_first=True,
    ):
        self.x_data = x_data
        self.y_data = y_data

        self.len_grid = x_data.shape[0]
        self.len_time = x_data.shape[1]

        self.buffer_time = 0

        self.batch_size = batch_size
        self.rho = rho

        assert stage_name in ["train", "valid", "test"]
        self.stage_name = stage_name
        self.num_valid = batch_size

        self.num_samples = self.get_samples_num()
        self.slice_grid_list, self.slice_time_list = self.get_samples_index(stage_name)

        self.batch_first = batch_first

    def __getitem__(self, i):

        slice_time = self.slice_time_list[i]
        slice_grid = self.slice_grid_list[i]

        # get data: basin, time, features
        """why choose this way: 
        https://stackoverflow.com/questions/71742745/how-to-quickly-select-a-sub-matrix-in-a-2-dimensional-matrix-using-numpy
        A shape of 1 for slice_grid is OK.
        """
        x_sub = self.x_data[slice_grid[:, None], slice_time, :]
        y_sub = self.y_data[slice_grid[:, None], slice_time, :]

        # change dim: time, basin, featues
        if not self.batch_first:
            x_sub = np.transpose(x_sub, [1,0,2])
            y_sub = np.transpose(y_sub, [1,0,2])

        # # apply preprocessing
        # if self.preprocessing:
        #     x,y = self.preprocessing(x,y)

        # convert to tensor
        x_sub = torch.from_numpy(x_sub).to(dtype=torch.float32)
        y_sub = torch.from_numpy(y_sub).to(dtype=torch.float32)

        return x_sub, y_sub, slice_grid, slice_time

    def __len__(self):
        return self.num_samples

    def get_samples_index(self, stage_name):
        slice_grid_list = []
        slice_time_list = []
        for idx in range(self.num_samples):
            if stage_name == "train":
                slice_grid, slice_time = self.random_sample_index(self.len_grid, self.len_time, self.batch_size, self.rho)
            else:
                slice_grid, slice_time = self.valid_sample_index(self.len_grid, self.len_time, idx)
            slice_grid_list.append(slice_grid)
            slice_time_list.append(slice_time)

        return slice_grid_list, slice_time_list

        # slice_grid, slice_time = self.random_samples_index(self.len_grid, self.len_time, self.batch_size, self.rho)
        # return slice_grid, slice_time

    def valid_sample_index(self, len_grid, len_time, idx):
        slice_time = np.arange(len_time).tolist()
        slice_grid = np.arange(len_grid)[idx*self.num_valid:(idx+1)*self.num_valid]

        return slice_grid, slice_time

    def random_sample_index(self, len_grid, len_time, batch_size, rho, buffer_time=0):
        iGrid = np.random.randint(0, len_grid, [batch_size])
        iT = np.random.randint(0 + buffer_time, len_time - rho, [batch_size])

        slice_grid = iGrid
        slice_time = [np.arange(x, x + rho).tolist() for x in iT]

        # slice data:
        # data[slice_grid[:,None], slice_time]

        return slice_grid, slice_time

    def get_samples_num(self):
        if self.stage_name == "train":
            num_samples = int(
                np.ceil(np.log(0.01) /
                        np.log(1 - self.batch_size * self.rho / self.len_grid / (self.len_time - self.buffer_time))))
        if self.stage_name in ["valid", "test"]:
            num_samples = int(np.ceil(self.len_grid / self.num_valid))
        return num_samples
