import numpy as np
import json
import os
from waterGPT.utils.data.camels2 import CAMELS

class LoadCamelsMultiple():

    def __init__(self):
        pass

    def load_all(self, inp_path, opt_path, train_date_list, valid_date_list, select_id_list):
        forcing_list = ["prcp", "srad", "tmax", "tmin", "vp"]  # try
        constant_list = ['elev_mean', 'slope_mean', 'area_gages2', 'frac_forest', 'lai_max', 'lai_diff', 'gvf_max',
                         'gvf_diff', 'soil_depth_pelletier', 'soil_depth_statsgo', 'soil_porosity', 'soil_conductivity',
                         'max_water_content', 'sand_frac', 'silt_frac', 'clay_frac', 'carbonate_rocks_frac',
                         'geol_permeability', 'p_mean', 'pet_mean', 'aridity', 'frac_snow', 'high_prec_freq',
                         'high_prec_dur', 'low_prec_freq', 'low_prec_dur']

        camles_nldas = CAMELS(inp_path=inp_path, opt_path=opt_path, select_id_list=select_id_list,
                              dataset="nldas_extended", forcing_list=forcing_list, constant_list=constant_list,
                              info_name="camles_info.json", )
        camles_maurer = CAMELS(inp_path=inp_path, opt_path=opt_path, select_id_list=select_id_list,
                               dataset="maurer_extended", forcing_list=forcing_list, constant_list=constant_list,
                               info_name="_.json")
        # from 1980 to 2008
        camles_daymet = CAMELS(inp_path=inp_path, opt_path=opt_path, select_id_list=select_id_list,
                               dataset="daymet", forcing_list=forcing_list, constant_list=constant_list,
                               info_name="_.json")

        train_x_nldas, train_c, train_y, stat_dict_nldas = camles_nldas.load_train(train_date_list=train_date_list,
                                                                                   convert_time_series=False)
        valid_x_nldas, valid_c, valid_y, obs = camles_nldas.load_test(test_date_list=valid_date_list,
                                                                      stat_dict=stat_dict_nldas,
                                                                      convert_time_series=False)

        train_x_maurer, _, _, stat_dict_maurer = camles_maurer.load_train(train_date_list=train_date_list,
                                                                          convert_time_series=False)
        valid_x_maurer, _, _, _ = camles_maurer.load_test(test_date_list=valid_date_list,
                                                          stat_dict=stat_dict_maurer,
                                                          convert_time_series=False)

        train_x_daymet, _, _, stat_dict_daymet = camles_daymet.load_train(train_date_list=train_date_list,
                                                                          convert_time_series=False)
        valid_x_daymet, _, _, _ = camles_daymet.load_test(test_date_list=valid_date_list,
                                                          stat_dict=stat_dict_daymet,
                                                          convert_time_series=False)

        # combine x and c
        train_c = train_c[:, None, :]
        train_c = np.repeat(train_c, train_x_nldas.shape[1], axis=1)
        train_x = np.concatenate([train_x_nldas, train_x_maurer, train_x_daymet, train_c], axis=-1)

        valid_c = valid_c[:, None, :]
        valid_c = np.repeat(valid_c, valid_x_nldas.shape[1], axis=1)
        valid_x = np.concatenate([valid_x_nldas, valid_x_maurer, valid_x_daymet, valid_c], axis=-1)

        # combine stat_dict_nldas
        for key in forcing_list:
            stat_dict_nldas[key + "_nldas"] = stat_dict_nldas.pop(key)
            stat_dict_maurer[key + "_maurer"] = stat_dict_maurer.pop(key)
            stat_dict_daymet[key + "_daymet"] = stat_dict_daymet.pop(key)

        stat_dict = {**stat_dict_nldas, **stat_dict_maurer, **stat_dict_daymet}
        var_times = [x + "_nldas" for x in forcing_list] + [x + "_maurer" for x in forcing_list] + [x + "_daymet" for x
                                                                                                    in forcing_list]
        var_const = constant_list

        # save stat_dict
        with open(os.path.join(opt_path, "stat_dict.json"), "w") as fout:
            json.dump(stat_dict, fout, indent=4)

        return train_x, train_y, valid_x, valid_y, obs, stat_dict, var_times, var_const