import torch
import numpy as np

def nanmax(x, dim=None):
    mask = torch.isnan(x)
    x[mask] = -torch.inf
    if not dim is None:
        max_val = torch.max(x, dim=dim)[0]
    else:
        max_val = torch.max(x)
    x[mask] = torch.nan
    return max_val


def nanmin(x, dim=None):
    mask = torch.isnan(x)
    x[mask] = torch.inf
    if not dim is None:
        max_val = torch.min(x, dim=dim)[0]
    else:
        max_val = torch.min(x)
    x[mask] = torch.nan
    return max_val


def convert_to_numpy(data):
    data_type = type(data)
    if data_type == torch.Tensor:
        return data.cpu().detach().numpy()
    else:
        return data


def covert_to_tensor(data):
    pass


# torchnp = TorchNP()

def nanstd(x, dim=1, keepdim=False, unbiased=False):
    """
    import numpy as np
    import torch
    np.std
    data = np.random.randn(3,4,2)
    data[0,0] = np.nan
    data[2,1] = np.nan
    print(np.nanstd(data, axis=1))
    data_torch = torch.from_numpy(data)
    print(nanstd(data_torch, time_dim=1))
    """
    time_dim = dim
    x_mean = torch.nanmean(x, dim=time_dim, keepdim=True)
    num = torch.sum(~torch.isnan(x), dim=time_dim, keepdim=True)
    if unbiased:
        num = num + 1
    tmp = (x - x_mean) ** 2
    nanvar = torch.nansum(tmp, dim=time_dim, keepdim=True) / num
    std = torch.sqrt(nanvar)

    if not keepdim:
        std = torch.squeeze(std)

    return std


def nanmedian(x, dim=None, keepdim=False):
    """
    The median is not unique for input tensors with an even number of elements. In this case the lower of the two medians
     is returned. To compute the mean of both medians, use torch.quantile() with q=0.5 instead.


    """
    if not dim is None:
        x = torch.nanquantile(x, q=0.5, keepdim=keepdim, dim=dim)
        x = torch.nanmedian(x, keepdim=keepdim, dim=dim)
    else:
        x = torch.flatten(x)
        x = torch.nanquantile(x, q=0.5)
        x = torch.nanmedian(x)
    return x
