import torch
import numpy as np

def align_dim(data, dim=None, sample=None):

    if not sample is None:
        dim = len(sample.shape)
    assert not dim is None
    data_dim = len(data.shape)
    while not data_dim == dim:
        if data_dim > dim:
            data = data[...,-1]
        elif data_dim < dim:
            data = data[...,None]
        else:
            return data
        data_dim = len(data.shape)
    return data

def convert_to_numpy(data):
    data_type = type(data)
    if data_type == torch.Tensor:
        return data.cpu().detach().numpy()
    else:
        return data

def covert_to_tensor(data):
    pass

import os
import json
import pandas as pd
import numpy as np
from collections import OrderedDict

def norm_streamflow():
    pass

def de_norm_streamflow():
    pass

def normalization(data, stat_list, from_raw=True,**kwargs):
    """

    data size: [..., features]. last dimension is features
    stat_list: [[90 percentile,10 percentile, mean,std], [90 percentile,10 percentile, mean,std]...]

    The feature order must correspond to the variable list order.

    from_raw = True
        normalization
    from_raw = False
        de_normalization

    """
    if data is None:
        return None

    if len(stat_list) == 1:
        data = cal_norm(data, stat=stat_list[0], from_raw=from_raw)
    else:
        for ndx, stat in enumerate(stat_list):
            data[...,ndx] = cal_norm(data[...,ndx], stat=stat, from_raw=from_raw)

    return data


def cal_norm(data, stat, from_raw=False):
    """
    e.g.
    data = np.arange(12).reshape(1, 3, 4),  # any order
    stat = [90 percentile, 10 percentile, mean, std]
    """
    if from_raw:
        # normalization
        data_out = (data - stat[2]) / stat[3]
    else:
        # de_normalization
        data_out = data * stat[3] + stat[2]
    return data_out


def load_stat(csv_folder=None, var_x_list=[], var_c_list=[], var_y_list=[],y_state_list=None):
    """
    e.g.
    var_timeseries = ["precipitation", "temperature"]
    vat_constant = ["soil", "DEM"]
    var_target = ["soil_moisture"]
    csv_path = "/data/format_data/")

    csv_path must include "statistics" subpath.

    stat_x_list, stat_y_list = load(csv_path, var_timeseries+var_constant, var_target)
    output:
        stat_x_list = {"precipitation":[left_p10, left_p90, mean, std],
                        "temperature":[left_p10, left_p90, mean, std],...]}
        stat_y_list: same as stat_x_list
        stat_list = {"x":stat_x_dict, "c":stat_c_dict, "y":stat_y_dict}

    """
    # Python 3.7 dicts in all Python implementations must preserve insertion order.
    # So you can use dict or OrderDict()

    stat_x_dict = OrderedDict()
    stat_c_dict = OrderedDict()
    stat_y_dict = OrderedDict()
    for var_s in var_x_list:
        csv_file = os.path.join(csv_folder, 'statistics', var_s + "_stat.csv")
        stat = pd.read_csv(csv_file, dtype=np.float, header=None).values.flatten()
        stat_x_dict[var_s] = stat.tolist()

    for var_s in var_c_list:
        csv_file = os.path.join(csv_folder, 'statistics', var_s + "_stat.csv")
        stat = pd.read_csv(csv_file, dtype=np.float, header=None).values.flatten()
        stat_c_dict[var_s] = stat.tolist()

    if not var_y_list is None:
        for var_s in var_y_list:
            csv_file = os.path.join(csv_folder, 'statistics', var_s + "_stat.csv")
            stat = pd.read_csv(csv_file, dtype=np.float, header=None).values.flatten()
            stat_y_dict[var_s] = stat.tolist()
    else:
        if y_state_list is None:
            stat_y_dict["y"] = [-9999., -9999., -9999., -9999]
        else:
            stat_y_dict["y"] = y_state_list

    stat_dict = {"x":stat_x_dict, "c":stat_c_dict, "y":stat_y_dict}

    return stat_dict


def batch_norma(data, ):
    """
    data size: [..., features]

    :param data:
    :return:
    """
    num_features = data.shape[-1]
    data_flatten = data.reshape([-1,num_features])
    mean = np.nanmean(data_flatten, axis=0)
    std = np.nanstd(data_flatten, axis=0)

    data_norm = (data - mean) / std
    return data_norm

def align_dim(data, dim=None, sample=None):

    if not sample is None:
        dim = len(sample.shape)
    assert not dim is None
    data_dim = len(data.shape)
    while data_dim == dim:
        if data_dim > dim:
            data = data[...,-1]
        elif data_dim < dim:
            data = data[...,None]
        else:
            return data
        data_dim = len(data.shape)
    return data


def de_normalization(data, stat_list, **kwargs):
    """

    data size: [..., features]. last dimension is features
    stat_list: [[90 percentile,10 percentile, mean,std], [90 percentile,10 percentile, mean,std]...]

    The feature order must correspond to the variable list order.


    """

    if len(stat_list) == 1:
        data = cal_de_norm(data, stat=stat_list[0])
    else:
        for ndx, stat in enumerate(stat_list):
            data[..., ndx] = cal_de_norm(data[..., ndx], stat=stat)

    return data


def cal_de_norm(data, stat):
    """
    e.g.
    data = np.arange(12).reshape(1, 3, 4),  # any order
    stat = [90 percentile, 10 percentile, mean, std]
    """
    return data * stat[3] + stat[2]
